/*****************************************************************************
    netio.h

    Diese Datei ist Teil des Programmpakets 'ACL-tftp'.
    Alle Rechte bei Andreas G. Schindler
                    Klappacher Str. 2i
                    64285 Darmstadt, Deutschland.

    Dieses Programm ist 'Freie Software'; Sie knnen es entsprechend den 
    Bedingungen der 'Allgemeinen ffentlichen GNU-Lizenz' (verffentlicht 
    durch die 'Free Software Foundation') weiter verbreiten und/oder 
    modifizieren. Massgeblich ist die Version 2 oder eine neuere Version
    dieser Lizenz. 

    Dieses Programm wird in der Hoffnung dass es ntzlich is weitergegeben,
    jedoch OHNE JEDE GARANTIE. Dieser Garantieauschluss bezieht sich auch
    auf Eigenschaften wie Marktfhigkeit oder Eignung zu einem bestimmten
    Zweck. Details finden sich in der 'Allgemeinen ffentlichen GNU-Lizenz'.

    Eine Kopie dieser Lizenz sollte in  Programmpaket, dem diese Datei
    entstammt, enthalten sein. Falls nicht, schreiben Sie an die 
    'Free Software Foundation, Inc.', 59 Temple Place, Suite 330, Boston, 
    MA  02111-1307  USA.


    This file is part of the 'ACL-tftp' Package
    Copyright (C) 1999-2005 by Andreas G. Schindler
                               Klappacher Str. 2i
                               Darmstadt, Germany.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*****************************************************************************/
#ifndef NETIO_H
#define NETIO_H

#ifndef _XOPEN_SOURCE_EXTENDED
#define _XOPEN_SOURCE_EXTENDED
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/stat.h>
#include "types.h"
#ifdef FILE_CACHE
#include "cache.h"
#endif

/*---	definitions taken from RFC 1350	and RFC 1782 ---*/

#define PE_ERRMASK	0xf

#define	PE_UNDEF	0		/* not defined */
#define	PE_NOTFOUND	1		/* file not found */
#define	PE_ACCESS	2		/* access violation */
#define	PE_NOSPACE	3		/* disk full or allocation exceeded */
#define	PE_BADOP	4		/* illegal TFTP operation */
#define	PE_BADID	5		/* unknown transfer ID */
#define	PE_EXISTS	6		/* file already exists */
#define	PE_NOUSER	7		/* no such user */
#define PE_ILLOPTVAL	8		/* illegal option value */

#define	PE_INTERNAL	0x10		/* internal server error */
#define PE_SYSTEM	0x20		/* system reported error */

#define PE_DISCARD	0x100		/* silent denial by ACL  */

#define	PX_UNDEF	PE_UNDEF   
#define	PX_NOTFOUND	PE_NOTFOUND
#define	PX_ACCESS	PE_ACCESS  
#define	PX_NOSPACE	PE_NOSPACE 
#define	PX_BADOP	PE_BADOP   
#define	PX_BADID	PE_BADID   
#define	PX_EXISTS	PE_EXISTS  
#define	PX_NOUSER	PE_NOUSER  
#define	PX_ILLOVAL	PE_ILLOVAL 
#define	PX_INTERNAL	PE_INTERNAL
#define	PX_SYSTEM	PE_SYSTEM  
#define	PX_DISCARD	PE_DISCARD

#define RT_RRQ		1		/* read request */
#define RT_WRQ		2		/* write request */
#define RT_DATA		3		/* data */
#define RT_ACK		4		/* acknowledgement */
#define RT_ERROR	5		/* error */
#define RT_OACK		6		/* option acknowlegement */

#define RR_SERVER	0		/* server operations	*/
#define RR_CLIENT	1		/* client operations	*/

#define RO_BSZ		"blksize"	/* block size option */
#define RO_TMO		"timeout"	/* timeout interval option */
#define RO_TSZ		"tsize"		/* transfer size option */
#define RO_BSZ2		"blksize2"	/* block size option */

#define ROT_BSZ		1		/* block size option */
#define ROT_TMO		2		/* timeout interval option */
#define ROT_TSZ		4		/* transfer size option */
#define ROT_BSZ2	8		/* power-of-2 block size option */

#define RO_BSZMIN	8
#define RO_BSZMAX	65536
#define RO_TMOMIN	2
#define RO_TMOMAX	3600
#define RO_TSZMIN	0		/* unused */
#define RO_TSZMAX	0xffffffff
#define RO_BSZ2MIN	8
#define RO_BSZ2MAX	32768

#define RMS_OCTET	"octet"
#define RMS_NETASCII	"netascii"
#define RMS_MAIL	"mail"		/* unsupported */

#define RM_OCTET	1
#define RM_NETASCII	2
#define RM_MAIL		3		/* unsupported */

#define RRO_SERVER	0		/* role of tftp-request struct */
#define RRO_CLIENT	1

#define SEGSIZE		512

typedef struct tftpH_s {	ushort	opCode;
				union {
					ushort blockno;
					short errcode;
					char  reqtext[1];
				 } th_u;			
					} tftpH_t;

typedef struct tftpD_s {	tftpH_t	head;
				char	data[SEGSIZE];

					} tftpD_t;
#define	blockNo	th_u.blockno
#define	errCode	th_u.errcode
#define reqText	th_u.reqtext

/*---	end of RFC 1350 definitions	---*/

#define	RES_CONFIG	1	/* allow name resolution in config stage */	
#define	RES_RUNTIME	2	/* allow name resolution at runtime      */	

typedef struct sockaddr    sa_t;
typedef struct sockaddr_in sai_t;
typedef struct timeval     utime_t;

typedef struct sopair_s {       int     recv;
                                int     send; } sopair_t;


typedef struct tout_s {		utime_t	send;	/* send completion      */
				utime_t	recv;	/* receive wait	        */
				utime_t	sack;	/* send ack completion  */
				utime_t	rack;	/* ack for receive wait */
				utime_t	idle;	/* idle wait            */
				utime_t	conn; 	/* connection setup     */
				utime_t	start;	/* start of xfer	*/
				utime_t	stop;	/* end of xfer		*/
				int	retry;	/* max. retry count	*/
				int	eflag;	/* special purpose flags*/
				double	xbfac; 	/* exp. backoff factors	*/

				} tout_t;

typedef struct opt_s  {	 	int	types;    /* client opts*/
				int	blocksize;/* max pack.	*/
				uint	totalsize;/* file size	*/	
				utime_t	timeout;  /* option tmo	*/	
				int	mandatory;/* options are mandatory */
				
				} opt_t;

typedef struct tftpR_s {	int	reqRole; /*client/server*/
				int	reqType; /* read/write 	*/
				int	reqMode; /* ascii/bin	*/
				int	reqPerm; /* perm-bitmask*/
				int	limGroup;/* thread limit group*/
				int	reqFd;   /* file descr.	*/
				char	*reqFile;/* file name	*/ /* dyn */
				char	*realFile;/* real file	*/ /* dyn */
				sopair_t reqSock;/* i/o sockets	*/
				sai_t	reqPeer; /* peer data	*/
				int	aclNo;   /* ACL number  */
				count_t	xfrstat; /* xfer bytes	*/
				count_t	iostat;  /* disc bytes	*/
				tout_t	times;   /* all timeouts*/	
				struct  varr_s	*filter; /* filter */ /* dyn */
				uint	maxsize; /* file size	*/	
				opt_t	option;  /* client opts	*/
				char	*bList;	 /* block list	*/ /* dyn */
				int	curBlock;/* curr. block */
				int	retry;	 /* retry count	*/
				int	lastAck; /* acked block */
#ifdef FILE_CACHE
				char	*fcache; /* file cache	*/
#endif
					} tftpR_t;

#define Rsock	reqSock.recv
#define Ssock	reqSock.send
#define hs(V)   ((V) & (0xffff))

void flushRequest (tftpR_t **request_p);
char *opCode2txt (int opcode);
uint setNetBuffer(int socket, uint minLen);
int setBits(in_addr_t mask);
void presetRequest (tftpR_t *request, int role, sopair_t *sockets);
int checkOptions(char *data, int len, tftpR_t *req);
char *appendOptions(char *buffer, tftpR_t *req, int *optlen);
int parseOptions (char *reqtext, int len, opt_t *op);
char *textOptions(tftpR_t *req);
double elapTime(tftpR_t *request);
double xfrRate(tftpR_t *request);
char *getErrText (tftpD_t *buf, int buflen);
void statusTrace (tftpR_t *request);
int name2sai (char *name, char *cport, sai_t *saip);
in_addr_t xinet_addr (char *host, int resolv);
char *xinet_ntoa(in_addr_t ia, int resolv);
char *xmask_ntoa(in_addr_t ia);
int setPort (char *cPort);
int isaSocketFd(int fd);
int isaSocket(FILE *fp);
int sendToError(int socket,sai_t *peer, int err, char *errText, tout_t *times);
int sendPeerError(tftpR_t *request, sai_t *peer, int err, char *errText);
int sendError(tftpR_t *request, int err, char *errText);
sopair_t *netStart(in_addr_t addr, int port);
int getRequest(sopair_t *sockets, tftpR_t **request_p);
int connectPeer(tftpR_t *request);
int recvFile(tftpR_t *request);
int sendFile(tftpR_t *request);
#endif /* NETIO_H */
